%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%           KINETIC MONTE CARLO MODELING             %%%%%%%%%%%%
%%%%%%%%%%%             OF AGGREGATE FORMATION                 %%%%%%%%%%%%
%%%%%%%%%%%            BY MULTIVALENT REACTANTS                %%%%%%%%%%%%
%%%%%%%%%%%               Author: A. S. Afshar                 %%%%%%%%%%%%
%%%%%%%%%%%            Last Modified: 09/26/2017               %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 1 - Parameter values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

rng('shuffle');

V        = 10^(-18);                % system volume (in m^3)
V0       = 1.5625* 10^(-20);        % voxel volume (in m^3) 
numVox   = 4;                       % number of voxels in each dimension
netVox   = numVox^3;                % total number of voxels
s        = nthroot(V/netVox,3);     % mesh spacing (in m)
nuL      = 5;                       % number of binding sites on molecule L for D 
nuP      = 5;                       % number of binding sites on molecule P for D
NL       = 200;                     % initial number of L molecules
NP       = 200;                     % initial number of P molecules
ND       = max(NL,NP)*max(nuL,nuP); % initial number of D molecules 
r        = 10^(-9);                 % reaction radius (in m)
d        = 10^(-11);                % diffusion coefficient (in m^2/s)
tmax     = 3;                       % simulation time (in sec)
Avogadro = 6.022 *10^23;            % Avogadro's number
AV_coef  = Avogadro*V0; 
i_val    = max(nuL,nuP);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 2 - Probability rates of RDME model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

k_plus      = [5.8*10^6 3.1*10^8 3.8*10^4 1.7*10^6]*(10^(-3));
k_minus     = [1.6*10^(-3) 3.1*10^(-5) 8.6*10^(-1) 1.9*10^(-2)];
kappa_plus  = (1/AV_coef)*k_plus; 
kappa_minus = k_minus;
kappas      = [kappa_plus kappa_minus];    % probability rates of reactions
lamdas_plus = (3/(4*pi))*((s/r)^3)*kappa_plus;                            
kd          = d/(s^2);                     % diffusion reaction rate                                 

%
% Check condition on voxel spacing
%

Slow  = max(r,((4*pi)/3)*((r^3)/d)*max(lamdas_plus));
Shigh = nthroot(V,3);

if s < 4*Slow || s > (1/4)*Shigh
   disp('***** RDME model NOT valid *****')
   return
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 3 - Initial molecular populations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% Total number of initial free binding sites.

numX11 = NL*nuL;
numX21 = NP*nuP;

% Allocation of voxel numbers to keep track of diffusion.

VoxM  = reshape(1:netVox,numVox,numVox,numVox);
VoxMD = zeros(numVox+2,numVox+2,numVox+2);
VoxMD(2:numVox+1,2:numVox+1,2:numVox+1) = VoxM;
VoxMD(1,2:numVox+1,2:numVox+1)= VoxM(1,:,:); 
VoxMD(numVox+2,2:numVox+1,2:numVox+1) = VoxM(numVox,:,:); 
VoxMD(2:numVox+1,1,2:numVox+1) = VoxM(:,1,:); 
VoxMD(2:numVox+1,numVox+2,2:numVox+1) = VoxM(:,numVox,:); 
VoxMD(:,:,1) = VoxMD(:,:,2);
VoxMD(:,:,numVox+2) = VoxMD(:,:,numVox+1);

% Initialization of status matrix and reactants for reactions.

S = zeros(5,numX11+2*ND+numX21);
S(1:2,:) = [1:numX11+2*ND+numX21;... 
ones(1,numX11) 2*ones(1,2*ND) 3*ones(1,numX21)];

Lvec = [];

for i = 1:NL
    Lvec = [Lvec i*ones(1,nuL)];
end

Dvec = [NL+1:NL+ND NL+1:NL+ND];

Pvec = [];
for i = NL+ND+1:NL+ND+NP
    Pvec = [Pvec i*ones(1,nuP)];
end

VoxSet = randsample(1:netVox,NL+ND+NP,true);

S(3,:) = [Lvec Dvec Pvec];
S(4,:) = VoxSet(S(3,:));
S(5,:) = S(3,:);

X11 = [1:numX11;ones(1,numX11);S(3,1:numX11);S(4,1:numX11)];
X12 = [numX11+1:numX11+ND;ones(1,ND);S(3,numX11+1:numX11+ND);...
       S(4,numX11+1:numX11+ND)];
X13 = zeros(5,numX11*ND);
X21 = [numX11+2*ND+1:numX11+2*ND+numX21;ones(1,numX21);...
       S(3,numX11+2*ND+1:numX11+2*ND+numX21);...
       S(4,numX11+2*ND+1:numX11+2*ND+numX21)];
X22 = [numX11+1:numX11+ND;zeros(1,ND);S(3,numX11+1:numX11+ND);...
       S(4,numX11+1:numX11+ND)];
X23 = zeros(5,numX11*ND);
X32 = [numX11+1+ND:numX11+2*ND;ones(1,ND);...
       S(3,numX11+1+ND:numX11+2*ND);...
       S(4,numX11+1+ND:numX11+2*ND)];
X33 = zeros(5,numX21*ND);
X42 = [numX11+1+ND:numX11+2*ND;zeros(1,ND);...
       S(3,numX11+1+ND:numX11+2*ND);...
       S(4,numX11+1+ND:numX11+2*ND)];
X43 = zeros(5,numX21*ND);

%
% Monte Carlo Initialization
%

iter          = 1;
t             = 0;
tau_d         = (s^2)/(6*d);
n_d           = 1;
Propensities  = zeros(netVox,8);
t_vec         = [];
tau_r_vec     = [];
iter_vec      = [];
r_num         = 0;
r_vec         = [];
d_num         = 0; 
rf_num        = 0;
rb_num        = 0;
agg_pres      = 0;
flag          = 0;
warning('off','all');
warning;
epochs        = 0:0.05:3;
epoch_index   = 1;
mols_size_vec = cell(1,length(epochs));
lp_sites      = [1:numX11 numX11+2*ND+1:numX11+2*ND+numX21];
agg_lth       = 5;
agg_hth       = 50;

% Main body of the RDME algorithm

while t <= tmax

    if t == 0
        [num_sites_temp, agg_ids_temp] = hist(S(5,lp_sites),unique(S(5,lp_sites)));
        num_mols_temp = num_sites_temp/i_val;
        mols_size_vec{epoch_index} = num_mols_temp;
        epoch_index = epoch_index + 1;
    end
        
    t_vec    = [t_vec t];
    iter_vec = [iter_vec iter];
    
    iter = iter+1;
    if mod(iter,50) == 0  
       iter
    end
    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 4 - Choose a voxel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

    while true
        
        for vox_id = 1:netVox
        
            % Identify number of reactants for each binding/unbinding 
            % reaction in each voxel
        
            nf_1 = sum(logical(X11(2,:))&X11(4,:)==vox_id )*...
                sum(logical(X12(2,:))&X12(4,:)==vox_id );
            nr_1 = sum(logical(X13(5,:))&X13(4,:)==vox_id );

            nf_2 = sum(logical(X11(2,:))&X11(4,:)==vox_id )*...
                sum(logical(X22(2,:))&X22(4,:)==vox_id );
            nr_2 = sum(logical(X23(5,:))&X23(4,:)==vox_id );

            nf_3 = sum(logical(X21(2,:))&X21(4,:)==vox_id )*...
                sum(logical(X32(2,:))&X32(4,:)==vox_id );
            nr_3 = sum(logical(X33(5,:))&X33(4,:)==vox_id );

            nf_4 = sum(logical(X21(2,:))&X21(4,:)==vox_id )*...
                sum(logical(X42(2,:))&X42(4,:)==vox_id );
            nr_4 = sum(logical(X43(5,:))&X43(4,:)==vox_id );

            Propensities(vox_id,:) = [nf_1 nf_2 nf_3 nf_4 nr_1 nr_2 nr_3 nr_4].*kappas;
        end
                
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % STEP 5 - Determine time of next reaction
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        
        astr      = sum(sum(Propensities));
        tau_r     = exprnd(inv(astr));
        tau_r_vec = [tau_r_vec tau_r];
        t         = t + tau_r;
        
        if t < tau_d*n_d && t >= epochs(epoch_index)        
            [num_sites_temp, agg_ids_temp] = hist(S(5,lp_sites),unique(S(5,lp_sites)));
            num_mols_temp = num_sites_temp/i_val;
            mols_size_vec{epoch_index} = num_mols_temp;
            epoch_index = epoch_index + 1;
        end
        
        if t > tmax
            break;
        end
        
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % STEP 6 - Determine reaction and corresponding voxel
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        
        vox_reac_id = find(cumsum(reshape(Propensities,[1,netVox*8])) > astr*rand,1);
        
        if mod(vox_reac_id,netVox)
            SelVox = mod(vox_reac_id,netVox);
            m = fix(vox_reac_id/netVox)+1;
        else
            SelVox = netVox;
            m = fix(vox_reac_id/netVox);
        end
        
        r_num = r_num + 1;
        
        if (m<5)
            rf_num=rf_num+1;
        end
        
        if (m>4) 
            rb_num=rb_num+1;
        end

        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % STEP 7 - Execute reaction
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %        
        
        switch m
            
            case 1
                RX11 = X11(1,logical(X11(2,:))&X11(4,:)==SelVox);
                x11  = RX11(unidrnd(length(RX11)));
                RX12 = X12(1,logical(X12(2,:))&X12(4,:)==SelVox);
                x12  = RX12(unidrnd(length(RX12)));
                X11(2,x11) = 0;
                X11(3,x11) = S(5,x11);
                X12(2,x12-numX11) = 0;
                X12(3,x12-numX11) = S(5,x11);
                X13(:,x11+numX11*(x12-numX11-1)) = [x11 x12 S(5,x11) SelVox 1];
                X13(3,X13(3,:)==S(5,x12)) = S(5,x11);

                X32(2,x12-numX11) = 0;
                X42([2 4],x12-numX11) = [1 SelVox];

                S(5,S(5,:)==S(5,x12)) = S(5,x11);

                X11(3,:) = S(5,1:numX11);
                X12(3,:) = S(5,numX11+1:numX11+ND);
                X22(3,:) = S(5,numX11+1:numX11+ND);
                X21(3,:) = S(5,numX11+2*ND+1:numX11+2*ND+numX21);
                X32(3,:) = S(5,numX11+1+ND:numX11+2*ND);
                X42(3,:) = S(5,numX11+1+ND:numX11+2*ND);

            case 2
                RX11 = X11(1,logical(X11(2,:))&X11(4,:)==SelVox);
                x11  = RX11(unidrnd(length(RX11)));
                RX22 = X22(1,logical(X22(2,:))&X22(4,:)==SelVox);
                x22  = RX22(unidrnd(length(RX22)));
                X11(2,x11) = 0;
                X11(3,x11) = S(5,x11);
                X22(2,x22-numX11) = 0;
                X22(3,x22-numX11) = S(5,x11);
                X23(:,x11+numX11*(x22-numX11-1)) = [x11 x22 S(5,x11) SelVox 1];
                X23(3,X23(3,:)==S(5,x22)) = S(5,x11);
                X13(:,x11+numX11*(x22-numX11-1)) = [x11 x22 S(5,x11) SelVox 0];

                X33(5,X33(1,:)==x22+ND) = 0;
                X43(:,X33(1,:)==x22+ND) = [x22+ND X33(2,X33(1,:)==x22+ND) S(5,x11) SelVox 1];

                X13(3,X13(3,:)==S(5,x22)) = S(5,x11);
                X23(3,X23(3,:)==S(5,x22)) = S(5,x11);
                X33(3,X33(3,:)==S(5,x22)) = S(5,x11);
                X43(3,X43(3,:)==S(5,x22)) = S(5,x11);

                S(5,S(5,:)==S(5,x22)) = S(5,x11);

                X11(3,:) = S(5,1:numX11);
                X12(3,:) = S(5,numX11+1:numX11+ND);
                X22(3,:) = S(5,numX11+1:numX11+ND);
                X21(3,:) = S(5,numX11+2*ND+1:numX11+2*ND+numX21);
                X32(3,:) = S(5,numX11+1+ND:numX11+2*ND);
                X42(3,:) = S(5,numX11+1+ND:numX11+2*ND);

            case 3
                RX21 = X21(1,logical(X21(2,:))&X21(4,:)==SelVox);
                x21  = RX21(unidrnd(length(RX21)));
                RX32 = X32(1,logical(X32(2,:))&X32(4,:)==SelVox);
                x32 = RX32(unidrnd(length(RX32)));
                x21_idx = x21-numX11-2*ND;
                x32_idx = x32-numX11-ND;
                X21(2,x21_idx) = 0;
                X21(3,x21_idx) = S(5,x21);
                X32(2,x32_idx) = 0;
                X32(3,x32_idx) = S(5,x21);
                X33(:,x21_idx+numX21*(x32_idx-1))=[x32 x21 S(5,x21) SelVox 1];
                X33(3,X33(3,:)==S(5,x32)) = S(5,x21);

                X12(2,x32_idx) = 0;
                X22([2 4],x32_idx) = [1 SelVox];
                S(5,S(5,:)==S(5,x32)) = S(5,x21);

                X11(3,:) = S(5,1:numX11);
                X12(3,:) = S(5,numX11+1:numX11+ND);
                X22(3,:) = S(5,numX11+1:numX11+ND);
                X21(3,:) = S(5,numX11+2*ND+1:numX11+2*ND+numX21);
                X32(3,:) = S(5,numX11+1+ND:numX11+2*ND);
                X42(3,:) = S(5,numX11+1+ND:numX11+2*ND);

            case 4
                RX21 = X21(1,logical(X21(2,:))&X21(4,:)==SelVox);
                x21  = RX21(unidrnd(length(RX21)));
                RX42 = X42(1,logical(X42(2,:))&X42(4,:)==SelVox);
                x42 = RX42(unidrnd(length(RX42)));
                x21_idx = x21-numX11-2*ND;
                x42_idx = x42-numX11-ND;
                X21(2,x21_idx) = 0;
                X21(3,x21_idx) = S(5,x21);
                X42(2,x42_idx) = 0;
                X42(3,x42_idx) = S(5,x21);
                X43(:,x21_idx+numX21*(x42_idx-1))=[x42 x21 S(5,x21) SelVox 1];
                X43(3,X43(3,:)==S(5,x42)) = S(5,x21);
                X33(:,x21_idx+numX21*(x42_idx-1))=[x42 x21 S(5,x21) SelVox 0];

                X13(5,X13(2,:)==x42-ND) = 0;
                X23(:,X13(2,:)==x42-ND) = [X13(1,X13(2,:)==x42-ND) x42-ND S(5,x21) SelVox 1];

                X13(3,X13(3,:)==S(5,x42)) = S(5,x21);
                X23(3,X23(3,:)==S(5,x42)) = S(5,x21);
                X33(3,X33(3,:)==S(5,x42)) = S(5,x21);
                X43(3,X43(3,:)==S(5,x42)) = S(5,x21);

                S(5,S(5,:)==S(5,x42)) = S(5,x21);

                X11(3,:) = S(5,1:numX11);
                X12(3,:) = S(5,numX11+1:numX11+ND);
                X22(3,:) = S(5,numX11+1:numX11+ND);
                X21(3,:) = S(5,numX11+2*ND+1:numX11+2*ND+numX21);
                X32(3,:) = S(5,numX11+1+ND:numX11+2*ND);
                X42(3,:) = S(5,numX11+1+ND:numX11+2*ND);

            case 5
                RX13 = X13(1:2,logical(X13(5,:))&X13(4,:)==SelVox);
                x13  = RX13(:,unidrnd(length(RX13(1,:))));
                X11([2 4],X11(1,:)==x13(1)) = [1 SelVox];
                X12(2:4,X12(1,:)==x13(2)) = [1 S(3,x13(2)) SelVox];
                X13(:,X13(2,:)==x13(2)) = [0 0 0 0 0];
                X22(3,X12(1,:)==x13(2)) = S(3,x13(2));
                X32(2:4,X12(1,:)==x13(2)) = [1 S(3,x13(2)) SelVox];
                X42(2:4,X12(1,:)==x13(2)) = [0 S(3,x13(2)) SelVox];
                S(5,S(3,:)==S(3,x13(2))) = S(3,x13(2));

            case 6
                RX23 = X23(1:2,logical(X23(5,:))&X23(4,:)==SelVox);
                x23  = RX23(:,unidrnd(length(RX23(1,:))));
                X11([2 4],X11(1,:)==x23(1)) = [1 SelVox];
                X22([2 4],X22(1,:)==x23(2)) = [1 SelVox];
                X33(5,X33(1,:)==x23(2)+ND) = 1;
                X23(:,X23(1,:)==x23(1)) = [0 0 0 0 0];
                X13(:,X13(1,:)==x23(1)) = [0 0 0 0 0];
                X43(:,X43(1,:)==x23(2)+ND) = [0 0 0 0 0];

                S(5,S(4,:)==SelVox) = S(3,S(4,:)==SelVox);

                for i = find(logical(X13(1,:))&X13(4,:)==SelVox)
                    S(5,S(5,:) == S(5,X13(2,i))) = S(5,X13(1,i));
                end

                for i = find(logical(X23(1,:))&X23(4,:)==SelVox)
                    S(5,S(5,:) == S(5,X23(2,i))) = S(5,X23(1,i));
                end

                for i = find(logical(X33(1,:))&X33(4,:)==SelVox)
                    S(5,S(5,:) == S(5,X33(1,i))) = S(5,X33(2,i));
                end

                for i = find(logical(X43(1,:))&X43(4,:)==SelVox)
                    S(5,S(5,:) == S(5,X43(1,i))) = S(5,X43(2,i));
                end

                for i = find(logical(X13(1,:))&X13(4,:)==SelVox)
                    X13(3,i) = S(5,X13(1,i));
                end

                for i = find(logical(X23(1,:))&X23(4,:)==SelVox)
                    X23(3,i) = S(5,X23(1,i));
                end

                for i = find(logical(X33(1,:))&X33(4,:)==SelVox)
                    X33(3,i) = S(5,X33(1,i));
                end

                for i = find(logical(X43(1,:))&X43(4,:)==SelVox)
                    X43(3,i) = S(5,X43(1,i));
                end

                X11(3,:) = S(5,1:numX11);
                X12(3,:) = S(5,numX11+1:numX11+ND);
                X22(3,:) = S(5,numX11+1:numX11+ND);
                X21(3,:) = S(5,numX11+2*ND+1:numX11+2*ND+numX21);
                X32(3,:) = S(5,numX11+1+ND:numX11+2*ND);
                X42(3,:) = S(5,numX11+1+ND:numX11+2*ND);

            case 7
                RX33 = X33(1:2,logical(X33(5,:))&X33(4,:)==SelVox);
                x33 = RX33(:,unidrnd(length(RX33(1,:))));
                X21([2 4],X21(1,:)==x33(2)) = [1 SelVox];
                X32(2:4,X32(1,:)==x33(1)) = [1 S(3,x33(1)) SelVox];
                X33(:,X33(1,:)==x33(1)) = [0 0 0 0 0];
                X12(2:4,X32(1,:)==x33(1)) = [1 S(3,x33(1)) SelVox];
                X22(2:4,X32(1,:)==x33(1)) = [0 S(3,x33(1)) SelVox];
                X42(3,X32(1,:)==x33(1)) = S(3,x33(1));
                S(5,S(3,:)==S(3,x33(1))) = S(3,x33(1));

            case 8
                RX43 = X43(1:2,logical(X43(5,:))&X43(4,:)==SelVox);
                x43 = RX43(:,unidrnd(length(RX43(1,:))));
                X21([2 4],X21(1,:)==x43(2)) = [1 SelVox];
                X42([2 4],X42(1,:)==x43(1)) = [1 SelVox];
                X13(5,X13(2,:)==x43(1)-ND) = 1;
                X43(:,X43(1,:)==x43(1)) = [0 0 0 0 0];
                X33(:,X33(1,:)==x43(1)) = [0 0 0 0 0];
                X23(:,X23(2,:)==x43(1)-ND) = [0 0 0 0 0];

                S(5,S(4,:)==SelVox) = S(3,S(4,:)==SelVox);

                for i = find(logical(X13(1,:))&X13(4,:)==SelVox)
                    S(5,S(5,:) == S(5,X13(2,i))) = S(5,X13(1,i));
                end

                for i = find(logical(X23(1,:))&X23(4,:)==SelVox)
                    S(5,S(5,:) == S(5,X23(2,i))) = S(5,X23(1,i));
                end

                for i = find(logical(X33(1,:))&X33(4,:)==SelVox)
                    S(5,S(5,:) == S(5,X33(1,i))) = S(5,X33(2,i));
                end

                for i = find(logical(X43(1,:))&X43(4,:)==SelVox)
                    S(5,S(5,:) == S(5,X43(1,i))) = S(5,X43(2,i));
                end

                for i = find(logical(X13(1,:))&X13(4,:)==SelVox)
                    X13(3,i) = S(5,X13(1,i));
                end

                for i = find(logical(X23(1,:))&X23(4,:)==SelVox)
                    X23(3,i) = S(5,X23(1,i));
                end

                for i = find(logical(X33(1,:))&X33(4,:)==SelVox)
                    X33(3,i) = S(5,X33(1,i));
                end

                for i = find(logical(X43(1,:))&X43(4,:)==SelVox)
                    X43(3,i) = S(5,X43(1,i));
                end

                X11(3,:) = S(5,1:numX11);
                X12(3,:) = S(5,numX11+1:numX11+ND);
                X22(3,:) = S(5,numX11+1:numX11+ND);
                X21(3,:) = S(5,numX11+2*ND+1:numX11+2*ND+numX21);
                X32(3,:) = S(5,numX11+1+ND:numX11+2*ND);
                X42(3,:) = S(5,numX11+1+ND:numX11+2*ND);
        end
           
        if t >= tau_d * n_d
            break;
        end
        
    end
    
    if t > tmax
        break;
    end
    
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % STEP 8 - Apply diffusion
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %    
            
    d_num  = d_num + 1;
    n_d    = n_d + 1;
    t      = n_d * tau_d;
    S_temp = S(4:5,:);
    
    for vox_id = 1:netVox 
        [r,c,h] = ind2sub([numVox numVox numVox],vox_id);
        DiffVec = [VoxMD(r,c+1,h+1) VoxMD(r+1,c,h+1) ...
            VoxMD(r+1,c+1,h) VoxMD(r+2,c+1,h+1) ...
            VoxMD(r+1,c+2,h+1) VoxMD(r+1,c+1,h+2)];
        SelVoxMolVec = unique(S(5,S(4,:)==vox_id));
        
        for j = 1:length(SelVoxMolVec)
            DiffMolNum = SelVoxMolVec(j);
            DiffVoxNum = DiffVec(unidrnd(6));

            S_temp(1,S_temp(2,:)==DiffMolNum) = DiffVoxNum;
            X11(4,X11(3,:)==DiffMolNum) = DiffVoxNum;
            X12(4,X12(3,:)==DiffMolNum) = DiffVoxNum;
            X13(4,X13(3,:)==DiffMolNum) = DiffVoxNum;
            X21(4,X21(3,:)==DiffMolNum) = DiffVoxNum;
            X22(4,X22(3,:)==DiffMolNum) = DiffVoxNum;
            X23(4,X23(3,:)==DiffMolNum) = DiffVoxNum;
            X32(4,X32(3,:)==DiffMolNum) = DiffVoxNum;
            X33(4,X33(3,:)==DiffMolNum) = DiffVoxNum;
            X42(4,X42(3,:)==DiffMolNum) = DiffVoxNum;
            X43(4,X43(3,:)==DiffMolNum) = DiffVoxNum;
        end
    end
    
    S(4:5,:) = S_temp;
    MolNum(iter) = numel(unique(S(5,:)));

    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % STEP 9 - Calculate and store results for subsequent analyses
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %  
    
    % Store molecular sizes
    
    if t>=epochs(epoch_index)
        [num_sites_temp, agg_ids_temp] = hist(S(5,lp_sites),unique(S(5,lp_sites)));
        num_mols_temp = num_sites_temp/i_val;
        mols_size_vec{epoch_index} = num_mols_temp;
        epoch_index = epoch_index +1;
    end
end

% Store certain metrics to track dynamics of simulation

fname = strcat('SizeDist_v',int2str(i_val),'_sample','.mat');
save(fname,'mols_size_vec','MolNum','t_vec','tau_r_vec','iter_vec','epoch_index');    
    
% Store the graph

agg = 0;
[num_sites,agg_ids] = hist(S(5,:),unique(S(5,:)));
large_agg_ids = agg_ids(num_sites > 5);

for agg_id_counter=1:length(large_agg_ids)
   agg_mol_id = large_agg_ids(agg_id_counter);
   agg_sites = S(1, S(5,:) == agg_mol_id);
   agg_lp_sites = agg_sites(agg_sites <= numX11 | agg_sites > numX11+2*ND);
   agg_size = numel(unique(S(3, agg_lp_sites)));

   if (agg_size >= agg_lth) && (agg_size <= agg_hth)
       agg = agg+1;
       [num_mol_sites,mol_comps] = hist(S(3, agg_lp_sites),unique(S(3, agg_lp_sites)));
       CL_num = 0;
       Ls = [];
       Ps = [];

       for i = S(1,agg_sites)
           CL_num = CL_num + sum(any(i==X23(2,:)) & any(i+ND==X43(1,:)));

           if (any(i==X23(2,:)) && any(i+ND==X43(1,:))) 
               Ls = [Ls X23(1,X23(2,:)==i)];
               Ps = [Ps X43(2,X43(1,:)==i+ND)]; 
           end
       end

       CLs = [Ls;Ps];
       lp_sites =[1:numX11 numX11+2*ND+1:numX11+2*ND+numX21];
       AdjMatdim = length(lp_sites)+length(CLs(1,:));

       G = zeros(AdjMatdim);
       Gverts = [lp_sites (numX11+2*ND+numX21+1):(numX11+2*ND+numX21+length(CLs(1,:)))];
       gvert_id = [Gverts; 1:AdjMatdim];

       for i = mol_comps
           mol_site_vec = S(1, S(3,:) == i);
           for j = 1:(length(mol_site_vec)-1)
               G(gvert_id(2, gvert_id(1,:) == mol_site_vec(j)), ... 
                gvert_id(2, gvert_id(1,:) == mol_site_vec(j+1))) = 1;
               G(gvert_id(2, gvert_id(1,:) == mol_site_vec(j+1)), ... 
                gvert_id(2, gvert_id(1,:) == mol_site_vec(j))) = 1;
           end
       end

        % Identify adjacent vertices for bound L-P sites, assign a new 
        % vertex label to the bound L-P sites, and connect the adjacent 
        % vertices to this new label. The range of the new labels is 
        % (numX11+2*ND+numX21+1):(numX11+2*ND+numX21+length(CLs(1,:)))
        % Remove the previous two labels by placing a zero in the 
        % corresponding row and column of the adjacency matrix.

       for i = 1:length(CLs(1,:))
           prev_neighbors = [find(G(gvert_id(2,gvert_id(1,:)==CLs(1,i)),:)) ...
               find(G(gvert_id(2,gvert_id(1,:)==CLs(2,i)),:))];

           for j = prev_neighbors
               G(j,gvert_id(2,gvert_id(1,:)==numX11+2*ND+numX21+i)) = 1;
               G(gvert_id(2,gvert_id(1,:)==numX11+2*ND+numX21+i),j) = 1;
           end

           for k = [gvert_id(2,gvert_id(1,:)==CLs(1,i)) ...
                   gvert_id(2,gvert_id(1,:)==CLs(2,i))]
               for j = find(G(k,:))
                   G(j,k) = 0;
                   G(k,j) = 0;
               end
           end
       end

       [r,c] = find(G);
       G_nonred = G;
       s_node_vec = find(accumarray(r, 1)==1);

       while (length(s_node_vec)~=0)
              for k = s_node_vec'
                  for j = find(G(k,:))
                      G(j,k) = 0;
                      G(k,j) = 0;
                  end
              end
             [r,c] = find(G);
             s_node_vec = find(accumarray(r, 1)==1);
       end

       sample = agg;
       
       % Store adjacency matrix of graph
       
       fname = strcat('G_v',int2str(i_val),'_sample_',int2str(sample),'.mat');
       save(fname,'G_nonred');

       [row,col] = find(G);
       A = [row col];
       
       % Store edge list of graph 
       
       fname2 = strcat('Gr_v',int2str(i_val),'_sample_',int2str(sample),'.txt');
       dlmwrite(fname2,A,'Delimiter','\t');

       fid = fopen(fname2,'a');
       fprintf(fid,'#');
       fclose(fid);
       
       % Store aggregate size and sample number
       
       fname3 = strcat('G_info_v',int2str(i_val),'_sample_',int2str(sample),'.mat');
       save(fname3,'agg_size','sample','t');
       
   end
    
end